/**
 *  \file client/Chat.h
 *  \brief Declaration of Chat class
 *
 *  $LastChangedBy: sjk114 $
 *  $Date: 2010-08-19 16:09:10 +1200 (Thu, 19 Aug 2010) $
 *
 */

#ifndef CHAT_H
#define CHAT_H

#include <iostream>
#include "TypeDef.h"
#include "Mutex.h"
#include "Subject.h"
#include "Outbox.h"

/**
 *  \class Chat
 *  \brief Stores a copy of the chat history and handles interaction
 */
class Chat : public Subject {
public:
  /**
   *  \brief Constructor
   *  \param id the ID of the chat to create
   *  \param outbox a pointer to the outbox for outbound communications
   */
  Chat(ChatID id, Outbox* outbox);
  
  /**
   * \brief Destructor
   */
  virtual ~Chat(){}
  
  /**
   *  \brief Tells the chat to recieve a message. 
   *         The chat will log the message and notify any observers (e.g. GUI).
   *  \param message The message to add to the chat history.
   */
  void Receive(Message message);

  /**
   * \brief Sends a message to the network connection.
   * \param message The message to be sent over the network connection.
   */
  void Send(Message message);
  
  /**
   *  \brief Leave the chat
   */
  void Leave(UserID user);
  
  /**
   *  \brief Invite a user to the chat
   *  \param user The user to invite
   */
  void Invite(UserID sender, UserID target);
  
  /**
   *  \return The chat history.
   */
  Messages const history();
  
  ChatID id();
  
  
private:
   
  //Attributes
  Messages history_;
  ChatID id_;
  Outbox* outbox_;
  
  //Thread safety
  Mutex mutex_;
};

#endif /* CHAT_H */
