/*
 *  ChatManager.cpp
 *  chat
 *
 *  Created by John Hofman on 29/09/10.
 *  Copyright 2010 University of Canterbury. All rights reserved.
 *
 */

#include "ChatManager.h"

ChatManager::ChatManager(Factory* factory, Outbox* outbox) {
  factory_ = factory;
  outbox_ = outbox;
}

void ChatManager::RequestChat(UserID sender, UserID target) {
  mutex_.Lock();
  outbox_->StartChat(sender, target);
  mutex_.Unlock();
}

void ChatManager::NewChat(ChatID id) {
  mutex_.Lock();
  chats_[id] = factory_->GetNewChat(id);
  mutex_.Unlock();
}

void ChatManager::EndChat(ChatID id) {
  mutex_.Lock();
  if(chats_[id] != NULL) {
    delete chats_[id];
    chats_.erase(id);
  }
  mutex_.Unlock();
}

void ChatManager::Receive(ChatID id, Message message) {
  mutex_.Lock();
  if(chats_[id] != NULL)
    chats_[id]->Receive(message);
  mutex_.Unlock();
}