/**
 *  \file  Factory.h
 *  \brief Abstract class defining a factory that can create chats and 
 *         user managers.
 *
 *  $LastChangedBy: sjk114 $
 *  $Date: 2010-08-19 14:38:13 +1200 (Thu, 19 Aug 2010) $
 *
 */

#ifndef FACTORY_H
#define FACTORY_H

#include "Chat.h"
#include "UserList.h"
#include "LocalUser.h"

//Forward declaration
class ChatManager;

/**
 *  \class Factory
 *  \brief Abstract class defining a factory that can create chats and 
 *         user managers.
 */
class Factory {
public:
  /**
   *  /brief Constructor
   */
  Factory(){}
  
  /**
   *  /brief Destructor
   */
  virtual ~Factory(){}
  
  /**
   *  \brief Make a new chat
   *  \param chatid The id of the new chat.
   *  \return the new chat
   */
  virtual Chat* GetNewChat(ChatID chatid) = 0;
  
  /**
   *  \brief Get the user manager
   *  \return the user manager.
   */
  virtual UserList* GetUserList() = 0;
  
  /**
   *  \brief Get the chat manager
   *  \return the chat manager
   */
  virtual ChatManager* GetChatManager() = 0;
  
  /**
   *  \brief Get the local user
   *  \return the local user
   */
  virtual LocalUser* GetLocalUser() = 0;  
};

#endif /* FACTORY_H */
