/**
 *  \file  Fl_Factory.h
 *  \brief The implementation of a FLTK factory for building GUI elements
 *
 *  $LastChangedBy: sjk114 $
 *  $Date: 2010-08-19 14:38:13 +1200 (Thu, 19 Aug 2010) $
 *
 */

#include "Fl_Factory.h"

Fl_Factory::Fl_Factory(Outbox* outbox) {
  outbox_ = outbox;
  intialised = false;
}

Chat* Fl_Factory::GetNewChat(ChatID chatid) {
  if(!intialised)
    Initialise();
  
  // Make the Chat and Fl_WindowedChat
  Chat* chat = new Chat(chatid, outbox_);
  
  Fl::lock();
  Fl_WindowedChat* w_chat = new Fl_WindowedChat(chat,
                                                local_user_,
                                                chat_manager_);
  Fl::unlock();
  
  //Register the Fl_WindowedChat as an observer of the Chat
  chat->Register(w_chat);
  
  //Return the chat
  return chat;
}

UserList* Fl_Factory::GetUserList() {
  if(!intialised)
    Initialise();
  
  return user_list_;
}

ChatManager* Fl_Factory::GetChatManager() {
  if(!intialised)
    Initialise();
 
  return chat_manager_;
}  

LocalUser* Fl_Factory::GetLocalUser() {
  if(!intialised)
    Initialise();
  
  return local_user_;
}

void Fl_Factory::Initialise() {
  user_list_ = new UserList(outbox_);
  local_user_ = new LocalUser(outbox_);
  chat_manager_ = new ChatManager(this, outbox_);
  
  Fl::lock();
  Fl_WindowedUser* w_user_manager = new Fl_WindowedUser(user_list_, 
                                                        local_user_, 
                                                        chat_manager_);
  Fl::unlock();
  
  user_list_->Register(w_user_manager);
  local_user_->Register(w_user_manager);
  chat_manager_->Register(w_user_manager);

  intialised = true;
}