/**
 *  \file  client/Inbox.h
 *  \brief Declaration of the client-side Inobx class
 *
 *  $LastChangedBy: sjk114 $
 *  $Date: 2010-08-19 16:09:10 +1200 (Thu, 19 Aug 2010) $
 *
 */

#ifndef INBOX_CLIENT_H
#define INBOX_CLIENT_H

#include "TypeDef.h"
#include "Factory.h"
#include "Thread.h"
#include "ClientSocket.h"
#include "ChatManager.h"
#include "LocalUser.h"
#include "UserList.h"

/**
 *  \class Inbox
 *  \brief Handles inbound communication from server to client
 */
class Inbox : public Thread {
public:
  /**
   *  \brief Constructor
   *  \param factory a pointer to the factory for creating chats and a user mgr
   *  \param socket a pointer to the socket to use for communications
   */
  Inbox(Factory* factory, Socket* socket);
  
  /**
   *  /brief Destructor
   */
  virtual ~Inbox();
  
  /**
   *  \brief Contains the tasks for the thread to execute. 
   */
  virtual void Run();
    
private:
  //Attributes
  Socket* socket_;
  ChatManager* chat_manager_;
  UserList* user_list_;
  LocalUser* local_user_;
};

#endif /* INBOX_CLIENT_H */
