/*
 *  LocalUser.h
 *  chat
 *
 *  Created by John Hofman on 29/09/10.
 *  Copyright 2010 University of Canterbury. All rights reserved.
 *
 */

#ifndef LOCALUSER_H
#define LOCALUSER_H

#include "Outbox.h"
#include "Mutex.h"
#include "TypeDef.h"
#include "Subject.h"


class LocalUser: public Subject {
public:
  /** States that the LocalUser can be in */
  enum State { LOGGED_OUT, LOGGING_IN, LOGGED_IN, DISCONNECTED};
  
  /**
   *  \brief Constructor
   *  \param outbox a pointer to the outbox for outbound communications
   */
  LocalUser(Outbox* outbox);
  
  /**
   *  \brief Destructor
   */
  ~LocalUser(){};
  
  /**
   *  \brief Attempt to login
   *  \param id The UserID to login in with.
   */
  void LoginAttempt(UserID id);
  
  /**
   *  \brief Notify that a login attempt was successful.
   *  \param id The UserID that was successfully logged in.
   */
  void LoginSuccess(UserID id);
  
  /**
   *  \brief Notify that a login attempt failed.
   */
  void LoginFailed();
  
  /**
   *  \brief Logout
   */
  void Logout();
  
  /**
   *  \brief Notify that the localuser has been disconnected from the server.
   */
  void Disconnect();
  
  /**
   *  \brief Get the currently logged in user ID.
   *  \returns The current id of the local user. An empty id if not logged in.
   */
  UserID id();
  
  /**
   *  \brief Get the current state of the localuser
   *  \return The state of the localuser.
   */
  State state();
  
private:
  Outbox* outbox_;
  UserID id_;
  State state_;
  Mutex mutex_;
};

#endif /*LOCALUSER_H*/