/**
 *  \file Mutex.cpp
 *  \brief Implementation of Mutex class. 
 *
 *  $LastChangedBy: jch231 $
 *  $Date: 2010-08-19 11:35:17 +1200 (Thu, 19 Aug 2010) $
 *
 */

#include "Mutex.h"

Mutex::Mutex() {
  //Initialise the mutex with default attributes.
  pthread_mutex_init(&mutex, NULL);
  pthread_cond_init(&condition_variable, NULL);
}

Mutex::~Mutex() {
  //Unlock and destroy the mutex.
  pthread_mutex_unlock(&mutex);
  pthread_mutex_destroy(&mutex);
  pthread_cond_destroy(&condition_variable);
}

void Mutex::Lock() {
  pthread_mutex_lock(&mutex);
}

void Mutex::Unlock() {
  pthread_mutex_unlock(&mutex);
}

void Mutex::Wait() {
  pthread_cond_wait(&condition_variable, &mutex);
}

void Mutex::Signal() {
  pthread_cond_signal(&condition_variable);
}

ScopedLock::ScopedLock(Mutex* mutex) {
  mutex_ = mutex;
  mutex_->Lock();
}

ScopedLock::~ScopedLock() {
  mutex_->Unlock();
}
  
