/**
 *  \file Subject.h
 *  \brief Declaration of an Subject class that implements the subject of
 *  the GOF Observer Design Pattern.
 *  chat
 *
 *  Created by John Hofman on 5/08/10.
 */

#ifndef SUBJECT_H
#define SUBJECT_H

#include <set>
#include "Observer.h"

/** \typedef Observers
 *  \brief A container of Observer references
 */
typedef std::set<Observer*> Observers;

/**
 *  \class Subject 
 *  \brief Encapsulates the subject of the GOF Observer Design Pattern. 
 *
 *  Maintains a record of all registered Observers and can notify them 
 *  when a change occurs.
 */
class Subject {
public:
  /**
   *  \brief Constructor
   */
  Subject(){}
  
  /**
   *  \brief Destructor
   */
  ~Subject(){}
  
  /**
   *  \brief Register an observer of this subject object.
   *  \param observer The observer to register.
   */
  void Register(Observer* observer);
  
  /**
   *  \brief Remove an observer of this subject object.
   *  \param observer The observer to remove.
   */
  void Deregister(Observer* observer);
  
  /**
   *  \brief Notify all observers.
   */
  void Notify();
  
private:
  //Record of registered observers.
  Observers observers;
};

#endif /* SUBJECT_H */
