/**
 *  \file  ChatManager.cpp
 *  \brief Implementation of a chat manager
 *
 *  $LastChangedBy: sjk114 $
 *  $Date: 2010-08-19 16:20:28 +1200 (Thu, 19 Aug 2010) $
 *
 */

#include "ChatManager.h"
#include <sstream>
#include <cstdio>

ChatManager::ChatManager(UserManager* user_manager) {
  user_manager_ = user_manager;
  chat_id_count = 0;
}


void ChatManager::Start(UserID user_sender, UserID user_target) {
  // Create a new Chat object with the next available ID
  mutex.Lock();
  std::stringstream temp;
  temp << chat_id_count; // Conversion from int to string
  chat_id_count++;
  ChatID id = temp.str();
  Chat* chat = new Chat(id, user_sender, user_target, user_manager_);
  chats_[id] = chat;
  mutex.Unlock();
}

void ChatManager::Invite(ChatID chat, UserID user_sender, UserID user_target) {
  mutex.Lock();
  // If the user isn't already in the chat, invite them
  if(chats_.find(chat) != chats_.end())
    chats_[chat]->Invite(user_sender, user_target);
  mutex.Unlock();
}

void ChatManager::Logout(UserID user) {
  // Notify every chat that the user was part of that they have logged out
  ChatMap::iterator it;
  for(it = chats_.begin(); it != chats_.end(); ++it) {
    it->second->Logout(user);
  }
}

void ChatManager::Leave(ChatID chat, UserID user) {
  // If the given chat exists, remove the user from it
  mutex.Lock();
  if(chats_.find(chat) != chats_.end())
    chats_[chat]->Leave(user);
  mutex.Unlock();
}


void ChatManager::Send(ChatID chat, Message message) {
  // If the given chat exists, send the message to it
  mutex.Lock();
  if(chats_.find(chat) != chats_.end())
    chats_[chat]->Receive(message);
  mutex.Unlock();
}

