/**
 *  \file server/Inbox.cpp
 *  \brief Implements the Inbox class for the server.
 *
 *  $LastChangedBy: sjk114 $
 *  $Date: 2010-08-19 16:09:10 +1200 (Thu, 19 Aug 2010) $
 *
 */

#include "Inbox.h"
#include "ChatManager.h"
#include "UserManager.h"

Inbox::Inbox(ServerSocket* socket, Outbox* outbox, UserManager* user_manager,
             ChatManager* chat_manager) {
  user_manager_ = user_manager;
  chat_manager_ = chat_manager;
  socket_ = socket;
  outbox_ = outbox;
}

void Inbox::Run(){

  UserID user;
  
  while(!waiting) {
    //Receive the packet
    Packet* packet = socket_->Receive();
  
    // Bail out if a packet couldn't be received
    // and logout the user if the user is logged in.
    if (packet == NULL) {
      if((user = outbox_->LoggedIn()) != "") {
        user_manager_->Logout(user);
        chat_manager_->Logout(user);
      }
      delete outbox_;
      delete socket_;
      break;
    }
    
    //Otherwise execute the comand in the packet.
    switch(packet->command()) {
      case LOGIN:
        user_manager_->Login(outbox_,packet->sender());
        break;
      case LOGOUT:
        if((user = outbox_->LoggedIn()) != "") {
          user_manager_->Logout(user);
          chat_manager_->Logout(user);
        }        
        delete outbox_;
        delete socket_;
        waiting = true;
        break;
      case STARTCHAT:
        chat_manager_->Start(packet->sender(), packet->target());
        break;
      case LEAVECHAT:
        chat_manager_->Leave(packet->chat(), packet->sender());
        break;
      case INVITE:
        chat_manager_->Invite(packet->chat(), packet->sender(), packet->target());
        break;
      case SEND:
        chat_manager_->Send(packet->chat(), packet->message());
        break;
      default:
        break;
    }
    
    delete packet;
  }
}

