/**
 *  \file Socket.cpp
 *  \brief Implementation of a Socket class that handles sending and recieving
 *  Packets to a socket.
 *
 *  $LastChangedBy: sjk114 $
 *  $Date: 2010-08-19 11:42:46 +1200 (Thu, 19 Aug 2010) $
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <string.h>
#include <iostream>
#include <string>
#include "Socket.h"

#define SIZE_TAG_LEN    4
#define SIZE_TAG_FORMAT "%04d"
#define BUFFER_SIZE     9999  /* = 10^SIZE_TAG_LEN - 1 */

const char* Socket::dataSeparator = " ";

void Socket::Send(Packet* data) {
  // Command -> char[] 
  char cmd[3];
  sprintf(cmd, "%d%s", data->command(), dataSeparator);
  
  // Serialise the Packet object to a space-separated string of tokens
  std::string data_string = "";
  data_string += cmd;
  data_string += data->chat()   + dataSeparator;
  if (data->message().text != "-")
    // If a Message is being sent, set the sender field to the message sender
    data_string += data->message().user + dataSeparator;
  else
    data_string += data->sender() + dataSeparator;
  data_string += data->target() + dataSeparator;
  data_string += data->message().text;
  data_string += "\0";
  
  // Set up a header indicating the length of the packet
  int packet_len = data_string.length();
  // TODO: actually handle this case - send multiple packets
  if (packet_len > BUFFER_SIZE) 
    printf("Your message is too long.\n");
  char packet_len_str[SIZE_TAG_LEN + 1];
  sprintf(packet_len_str, SIZE_TAG_FORMAT, packet_len);
  // Prepend the header to the data string
  data_string = std::string(packet_len_str) + data_string;
  
  // Write the string to the socket
  if (send(sock, data_string.c_str(), data_string.length(), 0) == -1) {
    perror("send");
  }
  
  // We're done with the Packet, so delete it
  delete data;
}

Packet* Socket::Receive() {
  int  recv_size;
  int  packet_len = 0;
  char recv_buffer[BUFFER_SIZE];
  Packet* packet = NULL;
  
  // Read the header to find out how long the following data is
  switch (recv_size = recv(sock, recv_buffer, SIZE_TAG_LEN, 0)) {
      case -1:
        perror("recv");
        break;
      case 0:
        std::cerr << "Connection closed by remote party." << std::endl;
        break;
      default:
        recv_buffer[recv_size] = '\0';
        // Parse the first 
        sscanf(recv_buffer, SIZE_TAG_FORMAT, &packet_len);
        break;
  }
  
  // Read data from the socket
  switch (recv_size = recv(sock, recv_buffer, packet_len, 0)) {
      case -1:
        perror("recv");
        break;
      case 0:
        std::cerr << "Connection closed by remote party." << std::endl;
        break;
      default:
        // Ensure string is null-terminated
        recv_buffer[recv_size] = '\0';
        
        std::cout << "Received: " << recv_buffer << std::endl;
      
        // Tokenise the string and build a Packet
        char* cmd = strtok(recv_buffer, dataSeparator);
        packet = new Packet(Command(*cmd - '0'));
        packet->chat(strtok(NULL, dataSeparator));
        packet->sender(strtok(NULL, dataSeparator));
        packet->target(strtok(NULL, dataSeparator));
        Message msg(packet->sender(), strtok(NULL, ""));
        packet->message(msg);  
  } 
  
  return packet;
}

Socket::~Socket() {
  // Close the socket
  close(sock);
}
