/**
 *  \file server/UserManager.cpp
 *  \brief Implements UserManager for the server
 *
 *  $LastChangedBy: sjk114 $
 *  $Date: 2010-08-19 16:09:10 +1200 (Thu, 19 Aug 2010) $
 *
 */

#include "UserManager.h"
#include <iostream>

UserManager::UserManager(){}

void UserManager::Login(Outbox* outbox_, UserID user) {
  mutex.Lock();
  //If the user is not already in the map
  if(users_.find(user) == users_.end()) {
    //Add the user
    users_[user] = outbox_;
    outbox_->LoginSuccess(user);
    
    //And push the update
    UserMap::iterator it;
    for(it = users_.begin(); it != users_.end(); ++it) {
      if((*it).first != user) {
        (*it).second->AddUser(user);
        outbox_->AddUser((*it).first);
      }
    }
  }
  else {
    //Otherwise notify the failure
    outbox_->LoginFailed();
  }
  mutex.Unlock();
    
}

void UserManager::Logout(UserID user) {
  mutex.Lock();
  //Remove the user
  users_.erase(user);
  UserMap::iterator it;
  //And push the update
  for(it = users_.begin(); it != users_.end(); ++it) {
    (*it).second->RemoveUser(user);
  }
  mutex.Unlock();
}

bool UserManager::Send(UserID user, ChatID chat, Message message) {
  bool result = true;
  mutex.Lock();
  //Attempt to send the message but return false if the user
  //isn't online.
  if(users_.find(user) == users_.end())
    result = false;
  else {
    users_[user]->Send(message, chat);
  }
  mutex.Unlock();
  return result;
}

bool UserManager::Start(UserID user, ChatID chat) {
  bool result = true;
  mutex.Lock();
  //Attempt to start the chat but return false if the user isn't
  //online
  if(users_.find(user) == users_.end())
    result = false;
  else
    users_[user]->StartChat(chat);
  
  mutex.Unlock();
  return result;
}


